<?php
/* --------------------------------------------------------------
   CustomerGroupCollectionSerializer.php 2017-10-04
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2017 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

/**
 * Class CustomerGroupCollectionSerializer
 *
 * @category   System
 * @package    CustomerGroup
 * @subpackage Serializers
 */
class CustomerGroupCollectionSerializer implements CustomerGroupCollectionSerializerInterface
{
	/**
	 * @var \CustomerGroupSerializerInterface
	 */
	protected $customerGroupSerializer;
	
	/**
	 * @var \CustomerGroupFactory
	 */
	protected $factory;
	
	
	public function __construct(CustomerGroupSerializerInterface $customerGroupSerializer,
	                            CustomerGroupFactory $factory)
	{
		$this->customerGroupSerializer = $customerGroupSerializer;
		$this->factory                 = $factory;
	}
	
	
	/**
	 * Serialize a value to a JSON string.
	 *
	 * @param \CustomerGroupCollection $collection Content to be serialized.
	 *
	 * @return array
	 */
	public function serialize(CustomerGroupCollection $collection)
	{
		$data = [];
		foreach($collection->getArray() as $customerGroup)
		{
			$data[] = $this->_serializeCustomerGroup($customerGroup);
		}
		
		return $data;
	}
	
	
	/**
	 * JSON Encode Wrapper.
	 *
	 * @param \CustomerGroupCollection $collection Content to be encoded.
	 *
	 * @return string Returns the encoded JSON string that represents the data.
	 */
	public function encode(CustomerGroupCollection $collection)
	{
		return json_encode($this->serialize($collection), JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
	}
	
	
	/**
	 * Deserialize a JSON string.
	 *
	 * @param \StringType $collection Content to be deserialize.
	 *
	 * @return \CustomerGroupCollection
	 */
	public function deserialize(StringType $collection)
	{
		$customerGroupData = json_decode($collection->asString(), true);
		
		$customerGroupCollection = $this->factory->createCollection();
		
		foreach($customerGroupData as $data)
		{
			$customerGroupEntity = $this->factory->createEntity();
			
			$customerGroupEntity->setId(new IntType($data['id']));
			
			foreach($data['names'] as $languageCode => $name)
			{
				$customerGroupEntity->setName(new StringType($name), new LanguageCode(new StringType($languageCode)));
			}
			
			$customerGroupEntity->setSettings($this->_deserializeCustomerGroupSettings($data['settings']));
			
			$customerGroupEntity->setConfigurations($this->_deserializeCustomerGroupConfigurations($data['configurations']));
			
			$customerGroupCollection->addItem($customerGroupEntity);
		}
		
		return $customerGroupCollection;
	}
	
	
	/**
	 * Serializes customer group entities.
	 *
	 * @param \CustomerGroupInterface $customerGroup Customer group entity to be serialized.
	 *
	 * @return array Serialized customer group array.
	 */
	protected function _serializeCustomerGroup(CustomerGroupInterface $customerGroup)
	{
		return [
			'id'             => $customerGroup->getId(),
			'names'          => $this->_serializeCustomerGroupNames($customerGroup->getNames()),
			'settings'       => $this->_serializeCustomerGroupSettings($customerGroup->getSettings()),
			'configurations' => $this->_serializeCustomerGroupConfigurations($customerGroup->getConfigurations())
		];
	}
	
	
	/**
	 * Serializes customer group names array.
	 *
	 * @param array $customerGroupNames Customer group names array to be serialized.
	 *
	 * @return array Serialized customer group names array.
	 */
	protected function _serializeCustomerGroupNames(array $customerGroupNames)
	{
		$data = [];
		foreach($customerGroupNames as $languageCode => $name)
		{
			$data[$languageCode] = $name;
		}
		
		return $data;
	}
	
	
	/**
	 * Serializes customer group configuration value objects.
	 *
	 * @param \CustomerGroupConfigurationsInterface $configurations Customer group configuration object to be
	 *                                                              serialized.
	 *
	 * @return array Serialized customer group configuration array.
	 */
	protected function _serializeCustomerGroupConfigurations(CustomerGroupConfigurationsInterface $configurations)
	{
		return [
			'minOrder'                 => $configurations->getMinOrder(),
			'maxOrder'                 => $configurations->getMaxOrder(),
			'discount'                 => $configurations->getDiscount(),
			'otDiscount'               => $configurations->getOtDiscount(),
			'unallowedPaymentModules'  => $configurations->getUnallowedPaymentModules(),
			'unallowedShippingModules' => $configurations->getUnallowedShippingModules()
		];
	}
	
	
	/**
	 * Serializes customer group settings value objects.
	 *
	 * @param \CustomerGroupSettingsInterface $settings Customer group settings object to be serialized.
	 *
	 * @return array Serialized customer group settings array.
	 */
	protected function _serializeCustomerGroupSettings(CustomerGroupSettingsInterface $settings)
	{
		return [
			'public'             => $settings->isPublic(),
			'otDiscountFlag'     => $settings->isOtDiscountFlag(),
			'graduatedPrices'    => $settings->isGraduatedPrices(),
			'showPrice'          => $settings->isShowPrice(),
			'showPriceTax'       => $settings->isShowPriceTax(),
			'addTaxOt'           => $settings->isAddTaxOt(),
			'discountAttributes' => $settings->isDiscountAttributes(),
			'fsk18'              => $settings->isFsk18(),
			'fsk18Display'       => $settings->isFsk18Display(),
			'writeReviews'       => $settings->isWriteReviews(),
			'readReviews'        => $settings->isReadReviews()
		];
	}
	
	
	/**
	 * Deserialize customer group settings.
	 *
	 * @param array $settings Customer group settings to be serialized.
	 *
	 * @return \CustomerGroupSettingsInterface Deserialize customer group settings.
	 */
	protected function _deserializeCustomerGroupSettings(array $settings)
	{
		return $this->factory->createSettings(new BoolType($settings['public']),
		                                      new BoolType($settings['otDiscountFlag']),
		                                      new BoolType($settings['graduatedPrices']),
		                                      new BoolType($settings['showPrice']),
		                                      new BoolType($settings['showPriceTax']),
		                                      new BoolType($settings['addTaxOt']),
		                                      new BoolType($settings['discountAttributes']),
		                                      new BoolType($settings['fsk18']), new BoolType($settings['fsk18Display']),
		                                      new BoolType($settings['writeReviews']),
		                                      new BoolType($settings['readReviews']));
	}
	
	
	/**
	 * Deserialize customer group configurations.
	 *
	 * @param array $configurations Customer group configurations to be serialized.
	 *
	 * @return \CustomerGroupConfigurationsInterface Deserialize customer group configurations.
	 */
	protected function _deserializeCustomerGroupConfigurations(array $configurations)
	{
		return $this->factory->createConfigurations(new DecimalType($configurations['discount']),
		                                            new DecimalType($configurations['otDiscount']),
		                                            $configurations['minOrder'] ? new DecimalType($configurations['minOrder']) : null,
		                                            $configurations['maxOrder'] ? new DecimalType($configurations['maxOrder']) : null,
		                                            $configurations['unallowedPaymentModules'],
		                                            $configurations['unallowedShippingModules']);
	}
}